@@ -102,6 +102,12 @@ struct var_list {
   msg_array *ary;
   var_list *next;
   VALUE file;
+  long disc;
+  long mtabl;
+  long cent;
+  long ltabl;
+  long categ;
+  long pnum;
 };
 static var_list*
 var_list_alloc(VALUE file)
@@ -150,7 +156,7 @@ push_msg_var(var_list **pvar, grib_handle *handle, VALUE file)
 {
   size_t len = MAX_VALUE_LENGTH;
   char vname[MAX_VALUE_LENGTH];
-  bzero(vname, len);
+  /*
   if (grib_get_string(handle, "shortName", vname, &len) != GRIB_SUCCESS ||
       strcmp("unknown", vname) == 0) {
     long id;
@@ -162,21 +168,48 @@ push_msg_var(var_list **pvar, grib_handle *handle, VALUE file)
       rb_raise(rb_eRuntimeError, "cannot get variable name");
     }
   }
+  */
+  long edition;
+  long cent, mtabl, pnum;
+  long disc, ltabl, categ;
   long ni, nj, ltype_id;
   char gtype[MAX_VALUE_LENGTH];
   len = MAX_VALUE_LENGTH;
-   if (grib_get_long(handle, "Ni", &ni) != GRIB_SUCCESS ||
-      grib_get_long(handle, "Nj", &nj) != GRIB_SUCCESS ||
-       grib_get_string(handle, "typeOfGrid", gtype, &len) != GRIB_SUCCESS ||
-       (grib_get_long(handle, "indicatorOfTypeOfLevel", &ltype_id) != GRIB_SUCCESS && grib_get_long(handle, "typeOfFirstFixedSurface", &ltype_id) != GRIB_SUCCESS)) {
+
+  grib_get_long(handle, "editionNumber", &edition);
+  grib_get_long(handle, "centre", &cent);
+  if(grib_get_long(handle, "Ni", &ni) != GRIB_SUCCESS ||
+     grib_get_long(handle, "Nj", &nj) != GRIB_SUCCESS ||
+     grib_get_string(handle, "typeOfGrid", gtype, &len) != GRIB_SUCCESS ||
+     (grib_get_long(handle, "indicatorOfTypeOfLevel", &ltype_id) != GRIB_SUCCESS && grib_get_long(handle, "typeOfFirstFixedSurface", &ltype_id) != GRIB_SUCCESS))
     rb_raise(rb_eRuntimeError, "connot identify grid type");
+  switch (edition) {
+  case 1:
+    if(grib_get_long(handle, "table2Version", &mtabl) != GRIB_SUCCESS ||
+       grib_get_long(handle, "indicatorOfParameter", &pnum) != GRIB_SUCCESS)
+      rb_raise(rb_eRuntimeError, "connot identify variable");
+    break;
+  case 2:
+    if (grib_get_long(handle, "tablesVersion", &mtabl) != GRIB_SUCCESS ||
+	grib_get_long(handle, "parameterNumber", &pnum) != GRIB_SUCCESS ||
+	grib_get_long(handle, "discipline", &disc) != GRIB_SUCCESS ||
+	grib_get_long(handle, "localTablesVersion", &ltabl) != GRIB_SUCCESS ||
+	grib_get_long(handle, "parameterCategory", &categ) != GRIB_SUCCESS)
+      rb_raise(rb_eRuntimeError, "connot identify variable");
   }
   var_list *last = NULL;
   var_list *var = pvar[0];
   if (var) {
     while (var) {
-      if (strcmp(var->vname, vname)==0 && var->ni == ni && var->nj == nj && strcmp(var->gtype,gtype)==0 && var->ltype_id==ltype_id) {
-	push_msg(var->ary, handle);
+      if (var->cent == cent && var->mtabl == mtabl && var->pnum == pnum &&
+	  var->ni == ni && var->nj == nj &&
+	  strcmp(var->gtype,gtype)==0 &&
+	  var->ltype_id==ltype_id) {
+	if (edition == 1)
+	  push_msg(var->ary, handle);
+	else if (edition == 2 &&
+	    var->disc == disc && var->ltabl == ltabl &&  var->categ == categ)
+	  push_msg(var->ary, handle);
 	return;
       }
       last = var;
@@ -186,14 +219,46 @@ push_msg_var(var_list **pvar, grib_handle *handle, VALUE file)
   var_list *var_new = var_list_alloc(file);
   if (var_new == NULL)
     rb_raise(rb_eRuntimeError, "cannot allocate memory");
+  var_new->cent = cent;
+  var_new->mtabl = mtabl;
+  var_new->pnum = pnum;
+  if (edition == 2) {
+    var_new->disc = disc;
+    var_new->ltabl = ltabl;
+    var_new->categ = categ;
+  }
   var_new->ni = ni;
   var_new->nj = nj;
   var_new->ltype_id = ltype_id;
-  strcpy(var_new->vname, vname);
   strcpy(var_new->gtype, gtype);
+
   len = MAX_VALUE_LENGTH;
+  bzero(vname, len);
+  if (grib_get_string(handle, "shortName", vname, &len) != GRIB_SUCCESS ||
+      strcmp("unknown", vname) == 0) {
+    sprintf(vname, "id%ld", pnum);
+  }
+  strcpy(var_new->vname, vname);
+
+  len = MAX_VALUE_LENGTH;
+  bzero(vname, len);
   check_error(grib_get_string(handle, "typeOfLevel", vname, &len));
-  strcpy(var_new->ltype, vname);
+  if (strcmp(vname, "unknown")==0 || strcmp(vname,"isobaricInhPa")== 0 || strcmp(vname,"pl")==0)
+    strcpy(var_new->ltype, vname);
+  else {
+    int i, j;
+    var_new->ltype[0] = vname[0];
+    for (i=1,j=1; i<strlen(vname); i++) {
+      if (vname[i]>=65 && vname[i]<=90) {
+	var_new->ltype[j] = vname[i]+32;
+	j++;
+      }
+    }
+    if (j==1)
+      strcpy(var_new->ltype, vname);
+    else
+      var_new->ltype[j] = 0;
+  }
   push_msg(var_new->ary, handle);
   if (last)
     last->next = var_new;
